==> forestry_1.11.2-5.3.0.5-sources.jar: forestry/arboriculture/blocks/BlockForestryFence.java <==
package forestry.arboriculture.blocks;

import java.util.Collection;

import forestry.api.arboriculture.IWoodType;
import forestry.api.arboriculture.TreeManager;
import forestry.api.arboriculture.WoodBlockKind;
import forestry.api.core.IItemModelRegister;
import forestry.api.core.IModelManager;
import forestry.api.core.IStateMapperRegister;
import forestry.api.core.Tabs;
import forestry.arboriculture.IWoodTyped;
import forestry.arboriculture.PluginArboriculture;
import forestry.arboriculture.WoodHelper;
import forestry.core.proxy.Proxies;
import net.minecraft.block.Block;
import net.minecraft.block.BlockFence;
import net.minecraft.block.BlockFenceGate;
import net.minecraft.block.BlockPlanks;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.ModelBakery;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public abstract class BlockForestryFence<T extends Enum<T> & IWoodType> extends BlockFence implements IWoodTyped, IItemModelRegister, IStateMapperRegister {
	protected static final int VARIANTS_PER_BLOCK = 16;

	private final boolean fireproof;
	private final int blockNumber;

	protected BlockForestryFence(boolean fireproof, int blockNumber) {
		super(Material.field_151575_d, BlockPlanks.EnumType.OAK.func_181070_c());
		this.fireproof = fireproof;
		this.blockNumber = blockNumber;

		PropertyWoodType<T> variant = getVariant();
		this.func_180632_j(this.field_176227_L.func_177621_b()
				.func_177226_a(field_176526_a, false)
				.func_177226_a(field_176525_b, false)
				.func_177226_a(field_176527_M, false)
				.func_177226_a(field_176528_N, false)
				.func_177226_a(variant, variant.getFirstType())
		);

		func_149711_c(2.0F);
		func_149752_b(5.0F);
		setHarvestLevel("axe", 0);
		func_149672_a(SoundType.field_185848_a);
		func_149647_a(Tabs.tabArboriculture);
	}

	public abstract PropertyWoodType<T> getVariant();

	@Override
	public boolean isFireproof() {
		return fireproof;
	}

	public int getBlockNumber() {
		return blockNumber;
	}

	@Override
	public boolean func_176524_e(IBlockAccess worldIn, BlockPos pos) {
		IBlockState blockState = worldIn.func_180495_p(pos);
		Block block = blockState.func_177230_c();
		if (PluginArboriculture.validFences.contains(block)) {
			return true;
		}

		if (block != Blocks.field_180401_cv) {
			Material blockMaterial = blockState.func_185904_a();
			if (block instanceof BlockFence || block instanceof BlockFenceGate) {
				return blockMaterial == this.field_149764_J;
			}
			if (blockMaterial.func_76218_k() && blockState.func_185917_h() && blockMaterial != Material.field_151572_C) {
				return true;
			}
		}
		return false;
	}

	/* Models */
	@SideOnly(Side.CLIENT)
	@Override
	public void registerModel(Item item, IModelManager manager) {
		ModelBakery.registerItemVariants(item, WoodHelper.getDefaultResourceLocations(this));
		manager.registerItemModel(item, new WoodHelper.WoodMeshDefinition(this));
	}

	@Override
	public float func_176195_g(IBlockState blockState, World worldIn, BlockPos pos) {
		int meta = func_176201_c(blockState);
		T woodType = getWoodType(meta);
		return woodType.getHardness();
	}

	@Override
	public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face) {
		if (fireproof) {
			return 0;
		}
		return 20;
	}

	@Override
	public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face) {
		if (fireproof) {
			return 0;
		}
		return 5;
	}

	@Override
	public WoodBlockKind getBlockKind() {
		return WoodBlockKind.FENCE;
	}

	@Override
	public int func_176201_c(IBlockState state) {
		return func_180651_a(state);
	}

	@Override
	public IBlockState func_176203_a(int meta) {
		T woodType = getWoodType(meta);
		return func_176223_P().func_177226_a(getVariant(), woodType);
	}

	@Override
	public abstract T getWoodType(int meta);

	@Override
	public Collection<T> getWoodTypes() {
		return getVariant().func_177700_c();
	}

	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, field_176526_a, field_176525_b, field_176528_N, field_176527_M, getVariant());
	}

	@Override
	public int func_180651_a(IBlockState state) {
		return state.func_177229_b(getVariant()).getMetadata() - blockNumber * VARIANTS_PER_BLOCK;
	}

	@Override
	public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
		T woodType = getWoodType(meta);
		return func_176223_P().func_177226_a(getVariant(), woodType);
	}

	@Override
	public void func_149666_a(Item itemIn, CreativeTabs tab, NonNullList<ItemStack> list) {
		for (T woodType : getVariant().func_177700_c()) {
			list.add(TreeManager.woodAccess.getStack(woodType, getBlockKind(), fireproof));
		}
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void registerStateMapper() {
		Proxies.render.registerStateMapper(this, new WoodTypeStateMapper(this, getVariant()));
	}
}

==> forestry_1.11.2-5.3.4.149-sources.jar: forestry/arboriculture/blocks/BlockForestryFence.java <==
package forestry.arboriculture.blocks;

import java.util.Collection;

import forestry.api.arboriculture.IWoodType;
import forestry.api.arboriculture.TreeManager;
import forestry.api.arboriculture.WoodBlockKind;
import forestry.api.core.IItemModelRegister;
import forestry.api.core.IModelManager;
import forestry.api.core.IStateMapperRegister;
import forestry.api.core.Tabs;
import forestry.arboriculture.IWoodTyped;
import forestry.arboriculture.PluginArboriculture;
import forestry.arboriculture.WoodHelper;
import forestry.arboriculture.proxy.ProxyArboricultureClient;
import net.minecraft.block.Block;
import net.minecraft.block.BlockFence;
import net.minecraft.block.BlockFenceGate;
import net.minecraft.block.BlockPlanks;
import net.minecraft.block.SoundType;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.renderer.block.model.ModelBakery;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public abstract class BlockForestryFence<T extends Enum<T> & IWoodType> extends BlockFence implements IWoodTyped, IItemModelRegister, IStateMapperRegister {
	protected static final int VARIANTS_PER_BLOCK = 16;

	private final boolean fireproof;
	private final int blockNumber;

	protected BlockForestryFence(boolean fireproof, int blockNumber) {
		super(Material.field_151575_d, BlockPlanks.EnumType.OAK.func_181070_c());
		this.fireproof = fireproof;
		this.blockNumber = blockNumber;

		PropertyWoodType<T> variant = getVariant();
		this.func_180632_j(this.field_176227_L.func_177621_b()
				.func_177226_a(field_176526_a, false)
				.func_177226_a(field_176525_b, false)
				.func_177226_a(field_176527_M, false)
				.func_177226_a(field_176528_N, false)
				.func_177226_a(variant, variant.getFirstType())
		);

		func_149711_c(2.0F);
		func_149752_b(5.0F);
		setHarvestLevel("axe", 0);
		func_149672_a(SoundType.field_185848_a);
		func_149647_a(Tabs.tabArboriculture);
	}

	public abstract PropertyWoodType<T> getVariant();

	@Override
	public boolean isFireproof() {
		return fireproof;
	}

	public int getBlockNumber() {
		return blockNumber;
	}

	@Override
	public boolean func_176524_e(IBlockAccess worldIn, BlockPos pos) {
		IBlockState blockState = worldIn.func_180495_p(pos);
		Block block = blockState.func_177230_c();
		if (PluginArboriculture.validFences.contains(block)) {
			return true;
		}

		if (block != Blocks.field_180401_cv) {
			Material blockMaterial = blockState.func_185904_a();
			if (block instanceof BlockFence || block instanceof BlockFenceGate) {
				return blockMaterial == this.field_149764_J;
			}
			if (blockMaterial.func_76218_k() && blockState.func_185917_h() && blockMaterial != Material.field_151572_C) {
				return true;
			}
		}
		return false;
	}

	/* Models */
	@SideOnly(Side.CLIENT)
	@Override
	public void registerModel(Item item, IModelManager manager) {
		ModelBakery.registerItemVariants(item, WoodHelper.getDefaultResourceLocations(this));
		ProxyArboricultureClient.registerWoodMeshDefinition(item, new WoodHelper.WoodMeshDefinition(this));
	}

	@Override
	public float func_176195_g(IBlockState blockState, World worldIn, BlockPos pos) {
		int meta = func_176201_c(blockState);
		T woodType = getWoodType(meta);
		return woodType.getHardness();
	}

	@Override
	public int getFlammability(IBlockAccess world, BlockPos pos, EnumFacing face) {
		if (fireproof) {
			return 0;
		}
		return 20;
	}

	@Override
	public int getFireSpreadSpeed(IBlockAccess world, BlockPos pos, EnumFacing face) {
		if (fireproof) {
			return 0;
		}
		return 5;
	}

	@Override
	public WoodBlockKind getBlockKind() {
		return WoodBlockKind.FENCE;
	}

	@Override
	public int func_176201_c(IBlockState state) {
		return func_180651_a(state);
	}

	@Override
	public IBlockState func_176203_a(int meta) {
		T woodType = getWoodType(meta);
		return func_176223_P().func_177226_a(getVariant(), woodType);
	}

	@Override
	public abstract T getWoodType(int meta);

	@Override
	public Collection<T> getWoodTypes() {
		return getVariant().func_177700_c();
	}

	@Override
	protected BlockStateContainer func_180661_e() {
		return new BlockStateContainer(this, field_176526_a, field_176525_b, field_176528_N, field_176527_M, getVariant());
	}

	@Override
	public int func_180651_a(IBlockState state) {
		return state.func_177229_b(getVariant()).getMetadata() - blockNumber * VARIANTS_PER_BLOCK;
	}

	@Override
	public IBlockState getStateForPlacement(World world, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer, EnumHand hand) {
		T woodType = getWoodType(meta);
		return func_176223_P().func_177226_a(getVariant(), woodType);
	}

	@Override
	public void func_149666_a(Item itemIn, CreativeTabs tab, NonNullList<ItemStack> list) {
		for (T woodType : getVariant().func_177700_c()) {
			list.add(TreeManager.woodAccess.getStack(woodType, getBlockKind(), fireproof));
		}
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void registerStateMapper() {
		ProxyArboricultureClient.registerWoodStateMapper(this, new WoodTypeStateMapper(this, getVariant()));
	}
}
